package com.wang.controller;

import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.WebResource;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.IOException;
import java.util.UUID;

//SpringMVC方式上传
@Controller
public class UploadController {
    //<input type="file" name="file"/>
    // MultipartFile参数名必须和JSP文件空间的name属性一致
    @RequestMapping("/fileUpload2")
    public String upload2(MultipartFile file, HttpServletRequest request) throws IOException {
        // 创建文件夹，存放上传文件
        String realPath = request.getSession().getServletContext().getRealPath("/upload");
        File dir = new File(realPath);
        //判断文件夹是否存在
        if (!dir.exists()) {
            dir.mkdirs();
        }
        // 将上传的数据写到文件夹的文件中
        // 1.拿到上传的文件名
        String filename = file.getOriginalFilename();
        //给文件名添加随机前缀
        filename = UUID.randomUUID() + "_" + filename;
        // 2.创建空文件
        File newFile = new File(dir, filename);
        // 3.将数据写入空文件中
        file.transferTo(newFile);

        return "index";
    }


    // 处理多文件上传,参数类型为MultipartFile数组，参数名和JSP文件控件的name属性一致
    //                              选择文件：<input type="file" name="file1"/>
    @RequestMapping("/fileUpload3")
    public String upload3(MultipartFile file1[], String username, HttpServletRequest request) throws Exception {
        System.out.println(username);
        //1.设置上传文件保存的文件夹 参数为文件夹名 upload
        String realPath = request.getSession().getServletContext().getRealPath("/upload");
        File dir = new File(realPath);
        if (!dir.exists()) {
            dir.mkdirs();
        }
        //2.遍历数组，将上传文件保存到文件夹
        for (MultipartFile file : file1) {
            // 1.拿到上传的文件名
            String filename = file.getOriginalFilename();
            //给文件名添加随机前缀
            filename = UUID.randomUUID() + "_" + filename;
            // 2.创建空文件
            File newFile = new File(dir, filename);
            // 3.将数据写入空文件中
            file.transferTo(newFile);
        }
        return "index";
    }


        /*之前的上传方案，在上传成功后都会跳转页面。而在实际开发中，很多情况下上传后不进行跳转，
        而是进行页面的局部刷新，比如：上传头像成功后将头像显示在网页中。这时候就需要使用异步文件上传。
        引入jQuery和jQuery表单上传工具jquery.form.js*/

    @RequestMapping("/fileUpload4")
    //不进行页面跳转 返回一段json数据 不经过试图解析器
    @ResponseBody
    public String upload3(HttpServletRequest request, MultipartFile file) throws Exception {
        // 创建文件夹，存放上传文件。
        String realPath = request.getSession().getServletContext().getRealPath("/upload");
        File dir = new File(realPath);
        if (!dir.exists()) {
            dir.mkdirs();
        }

        // 拿到上传文件名
        String filename = file.getOriginalFilename();
        filename = UUID.randomUUID() + "_" + filename;
        // 创建空文件
        File newFile = new File(dir, filename);
        // 将上传的文件写到空文件中
        file.transferTo(newFile);
        // 返回文件的路径
        return "/upload/" + filename;
    }

    /*由于文件占据磁盘空间较大，在实际开发中往往会将文件上传到其他服务器中，此时需要使用跨服务器上传文件。
    1。解压tomcat作为图片服务器，在tomcat的webapps下创建upload目录作为文件上传目录。
    2。修改tomcat的conf/web.xml文件，支持跨服上传。
    3。修改tomcat的conf/server.xml文件，修改tomcat端口，修改完开启tomcat服务器*/
    @RequestMapping("/fileUpload5")
    @ResponseBody
    public String upload4(HttpServletRequest request, MultipartFile file) throws Exception {
        // 设置跨服上传的服务器路径
        String path = "http://localhost:8081/upload/";
        // 获取上传的文件名
        String filename = file.getOriginalFilename();
        filename = UUID.randomUUID() + "_" + filename;

        // 跨服上传：
        // 1.创建客户端对象
        Client client = Client.create();
        // 2.使用客户端对象连接图片服务器
        WebResource resource = client.resource(path + filename);
        //3.传输数据
        resource.put(file.getBytes());

        return path + filename;
    }
}

