package com.wang.controller;

import com.wang.domain.Student;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.util.Date;
import java.util.List;

@Controller
public class MyController1 {
    /*返回值为void
    此时会跳转到名字是 前缀+方法路径名+后缀 的jsp页面*/

    //该控制器的访问路径为/c1/hello1
    @RequestMapping("/c1/hello1")
    public void helloMVC() {
        System.out.println("hello ");
    }


    /*在Servlet中我们通过request.getParameter(name)获取请求参数。该方式存在两个问题：
请求参数较多时会出现代码冗余。
与容器紧耦合。
而SpringMVC支持参数注入的方式用于获取请求数据，即将请求参数直接封装到方法的参数当中。用法如下：*/
    // 获取简单类型参数
    @RequestMapping("/c1/param1")
//访问该方法时，请求参数名和方法参数名相同，即可完成自动封装
//访问时 http://localhost:8080/c1/param1?username=qw&age=1 可以直接获取到参数
    public void simpleParam(String username, int age) {
        System.out.println(username);
        System.out.println(age);
    }

    @RequestMapping("/c1/param2")
    //SpringMVC支持将参数直接封装为对象
    public void obj(Student student) {
        System.out.println(student);
    }

    // 获取关联对象类型参数
    // http://localhost:8080/c1/param3?id=1&name=bz&sex=female&address.info=beijing&address.postcode=030000
    @RequestMapping("/c1/param3")
    public void objParam2(Student student) {
        System.out.println(student);
    }


    //表单发送带有参数的请求： param.jsp


    //SpringMVC支持将参数封装为List或Map集合

    // 绑定简单数据类型List参数，参数前必须添加@RequestParam注解
    //http://localhost:8080/c1/param4?users=bj&users=sxt
    @RequestMapping("/c1/param4")
    public void listParam(@RequestParam List<String> users) {
        System.out.println(users);
    }

    //该方式也可以绑定数组类型：
    @RequestMapping("/c1/param5")
    ////http://localhost:8080/c1/param4?users=bj&users=sxt
    public void listParam2(@RequestParam String[] users) {
        System.out.println(users[0]);
        System.out.println(users[1]);
    }

    //封装为对象类型集合
    //SpringMVC不支持将参数封装为对象类型的List集合，但可以封装到有List属性的对象中。
    // 对象中包含集合属性
    //http://localhost:8080/c1/param6?id=1&name=bz&sex=female&address[0].info=bj&address[0].postcode=100010&address[1].info=sh&address[1].postcode=100011
    @RequestMapping("/c1/param6")
    public void listParam3(Student student) {
        System.out.println(student);
    }

    //SpringMVC也支持使用Servlet原生对象，
    // 在方法参数中定义HttpServletRequest、HttpServletResponse、HttpSession等类型的参数即可直接在方法中使用。
    // http://localhost:8080/c1/param8?name=bjsxt
    //一般情况下，在SpringMVC中都有对Servlet原生对象的方法的替代，推荐使用SpringMVC的方式代替Servlet原生对象。
    @RequestMapping("/c1/param8")
    public void servletParam(HttpServletRequest request, HttpServletResponse response, HttpSession session) {
        // 原生对象获取参数
        System.out.println(request.getParameter("name"));
        System.out.println(response.getCharacterEncoding());
        System.out.println(session.getId());
    }
    @RequestMapping("/c1/param9")
public void dateParam(Date birthday){
            System.out.println(birthday);
        }

        /*在传递参数时，tomcat8以上能处理get请求的中文乱码，但不能处理post请求的中文乱码*/
    /*SpringMVC提供了处理中文乱码的过滤器，在web.xml中配置该过滤器即可解决中文乱码问题*/
    @RequestMapping("/cn/code")
public void code(String username){
           System.out.println(username);
        }
}
