package com.wang.controller;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.util.Map;

@Controller
public class MyController2 {
    /*返回值为void
    此时会跳转到名字是 前缀+方法路径名+后缀 的jsp页面*/
    @RequestMapping("/hello")
    public void helloww() {
        System.out.println("SpringMvc");
    }

    // 返回值为String
    //此时会跳转到名字是 前缀+返回值+后缀 的jsp页面
    @RequestMapping("/c2/hello1")
    public String helloMVC1() {
        System.out.println("hello SpringMVC！");
        // 方法执行完后会跳转到/hello.jsp
        return "hello";
    }

    /*这是SpringMVC提供的对象，该对象可以向request域设置数据并指定跳转的页面。
    该对象中包含Model对象和View对象。
    Model：向request域中设置数据。
    View：指定跳转的页面。*/
    // 返回值为ModelAndView
    @RequestMapping("/c2/hello2")
    public ModelAndView useMAV() {
        System.out.println("返回值类型为ModelAndView");
        // 1.创建ModelAndView对象
        ModelAndView modelAndView = new ModelAndView();
        // 2.获取Model对象，本质是一个Map
        Map<String, Object> model = modelAndView.getModel();
        // 3.使用Model对象向request域设置数据
        model.put("name", "百战程序员");
        // 4.使用View对象设置跳转的路径为/baizhan.jsp
        modelAndView.setViewName("wang");
        return modelAndView;
    }

    /*request域设置数据方法*/
    //1.使用原生的HttpServletRequest
    @RequestMapping("/c2/hello3")
    public String setRequestModel(HttpServletRequest request) {
        request.setAttribute("name", "尚学堂");
        return "wang";
    }

    //2.SpringMVC提供了Model接口和ModelMap类，控制器方法添加这两个类型的参数，
    // 使用该参数设置数据，该数据就会存到request域中。
    @RequestMapping("/c2/hello4")
    public String setRequestModel2(ModelMap modelMap) {
        // 使用ModelMap将数据存入request域
        modelMap.addAttribute("name", "尚学堂");
        return "wang";
    }

    @RequestMapping("/c2/hello5")
    public String setRequestModel2(Model model) {
        //使用Model将数据存入request域
        model.addAttribute("name", "尚学堂");
        return "wang";
    }

    @RequestMapping("/c2/hello6")
    public String setRequestModel3(Map map) {
        //使用Model将数据存入request域
        map.put("name", "尚学堂");
        return "wang";
    }


    //Session作用域表示在当前会话中有效。在SpringMVC中对于Session作用域传值，只能使用HttpSession对象来实现。
    @RequestMapping("/c2/hello7")
    public String setSessionModel(HttpSession session) {
        session.setAttribute("address", "北京");
        return "wang";
    }

    /*context作用域表示在整个应用范围都有效。在SpringMVC中对context作用域传值，只能使用ServletContext对象来实现。
    但是该对象不能直接注入到方法参数中，需要通过HttpSession对象获取。*/
    @RequestMapping("/c2/hello8")
    public String setContextModel(HttpSession session) {
        ServletContext servletContext = session.getServletContext();
        servletContext.setAttribute("age", 10);
        return "wang";
    }


    /*之前的案例，我们发现request域中的值可以传到jsp页面中，也就是通过视图解析器跳转到视图的底层是请求转发。
如果我们跳转时不想使用视图解析器，可以使用原生HttpServletRequest进行请求转发或HttpServletResponse进行重定向：*/

    /*不想使用试图解析器的情况：
    springmvc.xml中视图解析器配置的前缀为WEB-INF/page，但是你又想跳转到其他路径下的页面
    在不改变配置文件的情况下使用  请求转发forward:/c2/hello10 或者重定向redirect:/c2/hello10
    * */
    @RequestMapping("/c2/hello9")
    public void myForward1(HttpServletRequest request, HttpServletResponse response) throws Exception {
        request.setAttribute("name", "尚学堂");
        // 请求转发
        //        request.getRequestDispatcher("/c2/hello9").forward(request,response);
        // 原生重定向
        response.sendRedirect("/c2/hello10");
    }

    @RequestMapping("/c2/hello10")
    public void myForward2(HttpServletRequest request) {
        System.out.println("hello");
        System.out.println(request.getAttribute("name"));
    }

    //SpringMVC还提供了一种更简单的请求转发和重定向的写法
    @RequestMapping("/c2/hello11")
    public String myForward3(HttpServletRequest request) {
        request.setAttribute("name", "尚学堂");
        // 请求转发
        return "forward:/c2/hello10";
        // 重定向
        // return "redirect:/c2/hello10";
    }
}
